@@ -7,11 +7,14 @@ NULL
 #' @param y a vector of response variable.
 #' @param clin a matrix of clinical factors. It has default value NULL.
 #' @param X a matrix of genetic factors.
+#' @param W a matrix of random factors.
+#' @param nt a vector of number of repeated measurements for each subject. They can be same or different.
+#' @param group a vector of group sizes. They can be same or different.
 #' @param quant value of quantile.
 #' @param t0 a user-supplied sequence of the spike scale \eqn{s_{0}}.
 #' @param t1 a user-supplied sequence of the slab scale \eqn{s_{1}}.
 #' @param k number of folds for cross-validation.
-#' @param func methods to perform variable selection. Two choices are available: "ssLASSO" and "ssQLASSO".
+#' @param func methods to perform variable selection. Four choices are available. For non longitudinal analysis: "ssLASSO" and "ssQLASSO". For longitudinal varying-coefficient analysis: "ssVCM" and "ssQVCM".
 #' @param error cutoff value for determining convergence. The algorithm reaches convergence if the difference in the expected log-likelihood of two iterations is less than the value of error. The default value is 0.01.
 #' @param maxiter the maximum number of iterations that is used in the estimation algorithm. The default value is 200.
 #' @details
@@ -38,30 +41,61 @@ NULL
 #' 
 #' @export
 
-cv.emBayes <- function(y,clin=NULL,X,quant,t0,t1,k,func,error=0.01,maxiter=100){
+cv.emBayes <- function(y,clin=NULL,X,W=NULL,nt=NULL,group=NULL,quant,t0,t1,k,func,error=0.01,maxiter=100){
   l1 <- length(t0)
   l2 <- length(t1)
   n <- nrow(X)
   inter <- rep(1,n)
   C <- cbind(inter,clin)
   
-  s <- sample(1:n,n,replace=FALSE)
-  folds <- cut(s,breaks=k,labels=FALSE)
-  
   CV1 <- matrix(0,l1,l2)
   CV2 <- matrix(0,l1,l2)
   CV3 <- matrix(0,l1,l2)
-  for(i in 1:l1){
+  
+  if(grepl("VC",func)==0){
+    s <- sample(1:n,n,replace=FALSE)
+    folds <- cut(s,breaks=k,labels=FALSE)
+  
+    for(i in 1:l1){
+        for(j in 1:l2){
+          rob <- rep(0,k)
+          nrob <- rep(0,k)
+          SIC <- rep(0,k)
+          for(f in 1:k){
+            k.sub <- which(folds==f)
+            EM <- emBayes(y[-k.sub], clin[-k.sub,], X[-k.sub,],W=NULL,nt=NULL,group=NULL,quant,t0[i],t1[j],func,error,maxiter)
+            beta_k <- EM$beta
+            alpha_k <- EM$alpha
+            residual <- y[k.sub]-X[k.sub,]%*%beta_k-C[k.sub,]%*%alpha_k
+            edf <- sum(abs(residual) <= 1e-03)
+            rob[f] <- sum(residual*(quant-(residual < 0))) 
+            nrob[f] <- sum(residual^2)
+            SIC[f] <- log(rob[f])+log(n)*edf/(2*n)
+          }
+          CV1[i,j] <- sum(rob)/k
+          CV2[i,j] <- sum(nrob)/k
+          CV3[i,j] <- sum(SIC)/k
+        }
+    }
+  }
+  
+  else{
+    ns <- length(nt)
+    s <- sample(1:ns,ns,replace=FALSE)
+    folds <- cut(s,breaks=k,labels=FALSE)
+    
+    for(i in 1:l1){
       for(j in 1:l2){
         rob <- rep(0,k)
         nrob <- rep(0,k)
         SIC <- rep(0,k)
         for(f in 1:k){
           k.sub <- which(folds==f)
-          EM <- emBayes(y[-k.sub], clin[-k.sub,], X[-k.sub,],quant,t0[i],t1[j],func,error,maxiter)
+          EM <- emBayes(y[-k.sub], clin[-k.sub,], X[-k.sub,],W[-k.sub,],nt[-k.sub],group,quant,t0[i],t1[j],func,error,maxiter)
           beta_k <- EM$beta
           alpha_k <- EM$alpha
-          residual <- y[k.sub]-X[k.sub,]%*%beta_k-C[k.sub,]%*%alpha_k
+          r_k <- EM$r
+          residual <- y[k.sub]-X[k.sub,]%*%beta_k-C[k.sub,]%*%alpha_k-diag(W[k.sub,]%*%r_k)
           edf <- sum(abs(residual) <= 1e-03)
           rob[f] <- sum(residual*(quant-(residual < 0))) 
           nrob[f] <- sum(residual^2)
@@ -71,7 +105,9 @@ cv.emBayes <- function(y,clin=NULL,X,quant,t0,t1,k,func,error=0.01,maxiter=100){
         CV2[i,j] <- sum(nrob)/k
         CV3[i,j] <- sum(SIC)/k
       }
+    }
   }
+  
 
   indices1 <- which(CV1 == min(CV1), arr.ind=TRUE)
   indices2 <- which(CV2 == min(CV2), arr.ind=TRUE)
